@@ -46,18 +46,18 @@ struct CacheDisk;
 
 struct DiskVolBlock {
   uint64_t offset; // offset in bytes from the start of the disk
-  uint64_t len;    // length in in store blocks
+  uint64_t len;    // length in store blocks
   int number;
   unsigned int type : 3;
   unsigned int free : 1;
 };
 
 struct DiskVolBlockQueue {
-  DiskVolBlock *b;
-  int new_block; /* whether an existing vol or a new one */
+  DiskVolBlock *b = nullptr;
+  int new_block   = 0; /* whether an existing vol or a new one */
   LINK(DiskVolBlockQueue, link);
 
-  DiskVolBlockQueue() : b(nullptr), new_block(0) {}
+  DiskVolBlockQueue() {}
 };
 
 struct DiskVol {
